@@ -8,9 +8,9 @@ class Firestore:
     sub_collection_properties = 'properties'
     credentials = 'credentials'
 
-    def __init__(self, app=None):
+    def __init__(self, app=None, project_name = ul.G_CLOUD_PROJECT):
         self.firestore_db = firestore.client(app=get_app(app)) if app else firestore.client()
-        self.firestore_storage = storage.bucket(name=f'{ul.G_CLOUD_PROJECT}.appspot.com', app=get_app(app)) if app else storage.bucket(name=f'{ul.G_CLOUD_PROJECT}.appspot.com')
+        self.firestore_storage = storage.bucket(name=f'{project_name}.appspot.com', app=get_app(app)) if app else storage.bucket(name=f'{project_name}.appspot.com')
 
     def get_collection(self, name):
         return self.firestore_db.collection(name)
@@ -40,9 +40,15 @@ class Firestore:
             data: firestore.firestore.Increment(incr)
         })
 
-    def upload_document_firestorage(self, folder: str, imgData):
-        blob_ref = self.firestore_storage.blob(folder)
-        logging.debug(f"Blob Reference -> {blob_ref}")
-        blob_ref.upload_from_string(imgData)
-        # logging.debug(blob_ref.public_url)
+    def upload_document_firestorage(self, path: str, contentType: str,imgData: bytes, public: bool = False):
+        blob_ref = self.firestore_storage.blob(path)
+        blob_ref.upload_from_string(imgData, content_type=contentType)
+        if public:
+            blob_ref.make_public()
         return blob_ref.public_url
+
+    def delete_document_firestorage(self, path: str):
+        blob_ref = self.firestore_storage.blob(path)
+        if (blob_ref.exists()):
+            blob_ref.delete()
+        return True
