@@ -1,13 +1,16 @@
 import json
 from datetime import datetime
 
+from upswingutil.channel import trigger_reservation_email
+from upswingutil.channel.email import TriggerReservationEmailModel
 from upswingutil.db.model import Status, ReservationGuestInfo, ReservationGuestShort, \
     ReservationModelV2, GuestsModelV2
 from upswingutil.pms import store_reservation_to_alvie
 from upswingutil.resource import get_model_from_cloud_storage
 from upswingutil.pms.rms import NAME, get_key, validate_key
 from upswingutil.db import Mongodb, Firestore, MongodbV2
-from upswingutil.schema import Token
+from upswingutil.resource.user_management import create_and_update_app_user_profile
+from upswingutil.schema import Token, GuestUserProfileModel, BookingsModel
 import requests
 import logging
 from pycountry_convert import country_name_to_country_alpha2
@@ -166,6 +169,126 @@ def _retrieve_data(session, id, name, url):
         return name, result
 
 
+def _trigger_email(record: ReservationModelV2):
+    try:
+        if record.status == Status.CONFIRMED or record.status == Status.CREATED:
+            mongo = Mongodb(record.orgId)
+            _property = mongo.get_collection(mongo.PROPERTY_COLLECTION).find_one(
+                {'_id': record.idObj.hotelId, 'hotelId': record.idObj.hotelId})
+            __pipeline__ = [
+                {
+                    '$match': {
+                        '_id': record.id
+                    }
+                }, {
+                    '$unwind': {
+                        'path': '$guestInfo.guest_list'
+                    }
+                }, {
+                    '$lookup': {
+                        'from': 'guests',
+                        'localField': 'guestInfo.guest_list.guest',
+                        'foreignField': '_id',
+                        'as': 'guest_data'
+                    }
+                }, {
+                    '$unwind': {
+                        'path': '$guest_data'
+                    }
+                }, {
+                    '$project': {
+                        'guestId': '$guestInfo.guest_list.guest',
+                        'firstName': '$guest_data.firstName',
+                        'lastName': '$guest_data.lastName',
+                        'email': '$guest_data.contactInfo.email'
+                    }
+                }
+            ]
+            _guestDetailsList = mongo.execute_pipeline(mongo.RESERVATION_COLLECTION, __pipeline__)
+            if len(_guestDetailsList) > 0:
+                for _guest in _guestDetailsList:
+                    # '''
+                    # there may be more than one guest, we could have sent the email list of both guest
+                    # but the data for each guest such as firstname and lastname is different
+                    # '''
+                    if _guest['email'] != '':
+                        data = TriggerReservationEmailModel(
+                            orgId=record.orgId,
+                            hotelId=record.idObj.hotelId,
+                            hotelName=_property['name'],
+                            reservationId=record.id,
+                            firstName=_guest['firstName'],
+                            lastName=_guest['lastName'],
+                            guestEmail=_guest['email'],
+                            arrivalDate=record.datesAndDuration['arrivalDate'],
+                            departureDate=record.datesAndDuration['departureDate']
+                        )
+                        trigger_reservation_email(data)
+                    else:
+                        logging.error(f"Guest Email is empty for ResvId: {record.id}")
+            else:
+                logging.error(f"There is no guest details present in database for reservationId: {record.id}")
+    except Exception as e:
+        logging.error("Error Occured intriggering email")
+        logging.error(e)
+    finally:
+        mongo.close_connection()
+
+def _create_guestprofile(record: ReservationModelV2):
+    mongo = Mongodb(record.orgId)
+    __pipeline__ = [
+        {
+            '$match': {
+                '_id': record.id
+            }
+        }, {
+            '$unwind': {
+                'path': '$guestInfo.guest_list'
+            }
+        }, {
+            '$lookup': {
+                'from': 'guests',
+                'localField': 'guestInfo.guest_list.guest',
+                'foreignField': '_id',
+                'as': 'guest_data'
+            }
+        }, {
+            '$unwind': {
+                'path': '$guest_data'
+            }
+        }, {
+            '$project': {
+                'guestId': '$guestInfo.guest_list.guest',
+                'firstName': '$guest_data.firstName',
+                'lastName': '$guest_data.lastName',
+                'email': '$guest_data.contactInfo.email',
+                'mobile': '$guest_data.contactInfo.mobile'
+            }
+        }
+    ]
+    _guestDetailsList = mongo.execute_pipeline(mongo.RESERVATION_COLLECTION, __pipeline__)
+    if len(_guestDetailsList) > 0:
+        for _guest in _guestDetailsList:
+            booking_data = BookingsModel(hotelId=record.idObj.hotelId,reservationId = record.id,roomId= record.idObj.roomId)
+            data = GuestUserProfileModel(
+                profileSource='pms',
+                firstName=_guest['firstName'],
+                lastName=_guest['lastName'],
+                email=_guest['email'],
+                mobile=_guest['mobile'],
+                bookings = [booking_data]
+            )
+            resp = create_and_update_app_user_profile(record.orgId, data)
+            if resp.status:
+                logging.info(f"Created Profile successfully in User Management - resv: {record.id}")
+            else:
+                logging.error(f"Error in creating profile in User Management - resv: {record.id}")
+    else:
+        logging.error(f"Guests not added in reservation - {record.id}")
+
+    mongo.close_connection()
+
+
 class ReservationSync:
 
     def __init__(self, orgId: str, g_cloud_token=None):
@@ -238,7 +361,8 @@ class ReservationSync:
             logging.error(f'No nationalityCode found for {record.get("nationalityId")}')
             logging.error(e)
 
-        hotelId = self.mongo.get_collection(self.mongo.PROPERTY_COLLECTION).find_one({'primaryDetails.propertyId': str(record.get('propertyId'))}, {'_id': 1})
+        hotelId = self.mongo.get_collection(self.mongo.PROPERTY_COLLECTION).find_one(
+            {'primaryDetails.propertyId': str(record.get('propertyId'))}, {'_id': 1})
         hotelId = hotelId.get('_id') if hotelId else '0'
         guest = GuestsModelV2(
             _id=str(record.get('id')),
@@ -395,7 +519,8 @@ class ReservationSync:
             threads = []
             with ThreadPoolExecutor(max_workers=4) as executor:
                 for item in feature_list:
-                    threads.append(executor.submit(self._retrieve_data, record.get("id"), item.get("name"), item.get("url")))
+                    threads.append(
+                        executor.submit(self._retrieve_data, record.get("id"), item.get("name"), item.get("url")))
 
                 for th in as_completed(threads):
                     try:
@@ -494,7 +619,8 @@ class ReservationSync:
             if response.status_code == 200:
                 result = response.json()
             else:
-                logging.error(f'Error getting transactions for reservation {resvId} due to status code {response.status_code}')
+                logging.error(
+                    f'Error getting transactions for reservation {resvId} due to status code {response.status_code}')
                 logging.error(url.format(resvId))
                 logging.error(body)
         except Exception as e:
@@ -579,7 +705,8 @@ class ReservationSync:
                     'id': record.get('reservationTypes').get('id') if record.get('reservationTypes') else None,
                     'name': record.get('reservationTypes').get('name') if record.get('reservationTypes') else None,
                     'type': record.get('reservationType'),
-                    'inactive': record.get('reservationTypes').get('inactive') if record.get('reservationTypes') else None,
+                    'inactive': record.get('reservationTypes').get('inactive') if record.get(
+                        'reservationTypes') else None,
                     'fixedRes': record.get('fixedRes'),
                     'fixedResReasonId': record.get('fixedResReasonId'),
                 },
@@ -639,35 +766,36 @@ class ReservationSync:
                     'revenueBucketsInfo': None,
                     'transactions': self._get_transaction_info(record.get('id')),
                     'revenue': {
-                            'totalFixedCharge': {
-                                'amount': record.get('financial_info_actual').get('baseRate')
-                            },
-                            'totalPayment': {
-                                'amount': record.get('financial_info_actual').get('totalRate')
-                            },
-                            'roomRevenue': {
-                                'amount': sum([item['accommodation'] for item in record.get('daily_revenue')])
-                            },
-                            'foodAndBevRevenue': {
-                                'amount': sum([item['foodAndBeverage'] for item in record.get('daily_revenue')])
-                            },
-                            'otherRevenue': {
-                                'amount': sum([item['other'] for item in record.get('daily_revenue')]),
-                            },
-                            'totalRevenue': {
-                                'amount': sum([item['accommodation'] for item in record.get('daily_revenue')]) + sum(
-                                    [item['foodAndBeverage'] for item in record.get('daily_revenue')]) + sum(
-                                    [item['other'] for item in record.get('daily_revenue')])
-                            },
-                            'balance': {
-                                'amount': record.get('financial_info_actual').get('accommodationBalance')
-                            }
+                        'totalFixedCharge': {
+                            'amount': record.get('financial_info_actual').get('baseRate')
                         },
+                        'totalPayment': {
+                            'amount': record.get('financial_info_actual').get('totalRate')
+                        },
+                        'roomRevenue': {
+                            'amount': sum([item['accommodation'] for item in record.get('daily_revenue')])
+                        },
+                        'foodAndBevRevenue': {
+                            'amount': sum([item['foodAndBeverage'] for item in record.get('daily_revenue')])
+                        },
+                        'otherRevenue': {
+                            'amount': sum([item['other'] for item in record.get('daily_revenue')]),
+                        },
+                        'totalRevenue': {
+                            'amount': sum([item['accommodation'] for item in record.get('daily_revenue')]) + sum(
+                                [item['foodAndBeverage'] for item in record.get('daily_revenue')]) + sum(
+                                [item['other'] for item in record.get('daily_revenue')])
+                        },
+                        'balance': {
+                            'amount': record.get('financial_info_actual').get('accommodationBalance')
+                        }
+                    },
                     'deposit': {
                         'deposit': record.get('financial_info_actual').get('deposit'),
                         'depositRequiredByDate': record.get('financial_info_actual').get('depositRequiredByDate'),
                         'secondDeposit': record.get('financial_info_actual').get('secondDeposit'),
-                        'secondDepositRequiredByDate': record.get('financial_info_actual').get('secondDepositRequiredByDate'),
+                        'secondDepositRequiredByDate': record.get('financial_info_actual').get(
+                            'secondDepositRequiredByDate'),
                     },
                     'discount': [
                         {
@@ -683,7 +811,8 @@ class ReservationSync:
                         }
                     ],
                     'commission': {
-                        'travelAgentCommissionPercentage': record.get('financial_info_actual').get('travelAgentCommissionPercentage')
+                        'travelAgentCommissionPercentage': record.get('financial_info_actual').get(
+                            'travelAgentCommissionPercentage')
                     },
                     # 'billTo': record.get('billTo'),
                     # 'billing': record.get('billing')
@@ -741,7 +870,8 @@ class ReservationSync:
                      'hotelId': 1}
                 )
                 response_json['clientId'] = property_info.get('hotelId') if property_info else 0
-                response_json['propertyId'] = property_info.get('primaryDetails').get('propertyId') if property_info else 0
+                response_json['propertyId'] = property_info.get('primaryDetails').get(
+                    'propertyId') if property_info else 0
                 response_json['propertyName'] = property_info.get('name') if property_info else 'Unknown'
                 return response_json
             except Exception as err:
@@ -768,6 +898,8 @@ class ReservationSync:
             if record:
                 self.mongoV2.save(record)
                 store_reservation_to_alvie(record)
+                _create_guestprofile(record)
+                _trigger_email(record)
             else:
                 logging.error(f'Error saving the record {record} as it is None')
 
